/*  colortail -- output last part of file(s) in color
    Copyright (C) 1999  Joakim Andersson <pt98jan@student.hk-r.se>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <config.h>

#include "cfg.h"
#include "main.h"

#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <regex.h>

int parse_cfgfile(FILE *file) {
   /* parses the ctffile and sets up the global list(s)
      return values:
      number of search_data items created
   */
   char tmp[MAX_LINE_LENGTH];
   int loop, loop2, linepos, nr = 0, found_color;
   char color[20];
   struct search_data *item;
   void (*fkn)(char*);
   char *fkn_param;

   if (!file) {
      return nr;
   }
   
   loop = 1;
   while (loop) {
      /* reset variables */
      fkn = NULL;
      fkn_param = NULL;
      
      if (fgets(tmp, MAX_LINE_LENGTH - 1, file) == NULL) {
	 return nr;
      }
      
      /* skip if nothing or a # */
      if (tmp[0] != '\n' && tmp[0] != '#') {
	 if (strncmp(tmp, "COLOR", 5) != 0) {
	    fprintf(stderr, "Error in config file 1 : %s\n", tmp);
	    return nr;
	 }
	 linepos = 5;
	 while (tmp[linepos] == ' ') {
	    linepos++;
	 }
	 
	 found_color = 0;
	 
	 if (strncmp(&tmp[linepos], "black", 5) == 0) {
	    strcpy(color, "\033[0;30m");
	    found_color = 1;
	 }

	 if (strncmp(&tmp[linepos], "red", 3) == 0) {
	    strcpy(color, "\033[0;31m");
	    found_color = 1;
	 }
	 
	 if (strncmp(&tmp[linepos], "green", 4) == 0) {
	    strcpy(color, "\033[0;32m");
	    found_color = 1;
	 }

	 if (strncmp(&tmp[linepos], "yellow", 6) == 0) {
	    strcpy(color, "\033[0;33m");
	    found_color = 1;
	 }

	 if (strncmp(&tmp[linepos], "blue", 4) == 0) {
	    strcpy(color, "\033[0;34m");
	    found_color = 1;
	 }

	 if (strncmp(&tmp[linepos], "magenta", 7) == 0) {
	    strcpy(color, "\033[0;35m");
	    found_color = 1;
	 }

	 if (strncmp(&tmp[linepos], "cyan", 4) == 0) {
	    strcpy(color, "\033[0;36m");
	    found_color = 1;
	 }

	 if (strncmp(&tmp[linepos], "white", 6) == 0) {
	    strcpy(color, "\033[0;37m");
	    found_color = 1;
	 }

	 if (strncmp(&tmp[linepos], "brightblack", 6+5) == 0) {
	    strcpy(color, "\033[1;30m");
	    found_color = 1;
	 }

	 if (strncmp(&tmp[linepos], "brightred", 6+3) == 0) {
	    strcpy(color, "\033[1;31m");
	    found_color = 1;
	 }

	 if (strncmp(&tmp[linepos], "brightgreen", 6+5) == 0) {
	    strcpy(color, "\033[1;32m");
	    found_color = 1;
	 }

	 if (strncmp(&tmp[linepos], "brightyellow", 6+6) == 0) {
	    strcpy(color, "\033[1;33m");
	    found_color = 1;
	 }

	 if (strncmp(&tmp[linepos], "brightblue", 6+4) == 0) {
	    strcpy(color, "\033[1;34m");
	    found_color = 1;
	 }

	 if (strncmp(&tmp[linepos], "brightmagenta", 6+7) == 0) {
	    strcpy(color, "\033[1;35m");
	    found_color = 1;
	 }

	 if (strncmp(&tmp[linepos], "brightcyan", 6+4) == 0) {
	    strcpy(color, "\033[1;36m");
	    found_color = 1;
	 }

	 if (strncmp(&tmp[linepos], "brightwhite", 6+5) == 0) {
	    strcpy(color, "\033[1;37m");
	    found_color = 1;
	 }
	 
	 if (found_color == 0) {
	    fprintf(stderr, "Error in config file 2\n");
	    return nr;
	 }
	 
	 if (fgets(tmp, MAX_LINE_LENGTH - 1, file) == NULL) {
	    fprintf(stderr, "Error in config file 3\n");
	    return nr;
	 }

	 if (strncmp(tmp, "ACTION", 6) == 0) {
	    linepos = 6;
	    while (tmp[linepos] == ' ') {
	       linepos++;
	    }

	    if (strncmp(&tmp[linepos], "beep", 4) == 0) 
	    {
	       fkn = beep;
	    }
	    else 
	    {
	       /* don't need to do anything, because of = NULL at the top */
	    }
	    
	    if (fgets(tmp, MAX_LINE_LENGTH - 1, file) == NULL) {
	       fprintf(stderr, "Error in config file 4\n");
	       return nr;
	    }
	 }
	 
	 if (tmp[0] != '{') {
	    fprintf(stderr, "Error in config file 5\n");
	    return nr;
	 }
	 
	 /* loop2 = 1; */
	 while (1) {
	    if (fgets(tmp, MAX_LINE_LENGTH - 1, file) == NULL) {
	       fprintf(stderr, "Error in config file 6\n");
	       return nr;
	    }
	    
	    if (tmp[0] == '}') {
	       /*loop2 = 0;*/
	       break;
	    }

	    /* ignore empty lines */
	    if (tmp[0] != '\0' || tmp[0] != '\n') {	       
	       int l = strlen(tmp);

	       /* remove \n a end of string */
	       if (tmp[l-1] == '\n') {
		  tmp[l-1] = tmp[l];
		  l--;
	       }

	       /* TODO malloc error checking */
	       
	       item = (struct search_data *) malloc(sizeof(struct search_data));

	       item->preg = (regex_t *) malloc(sizeof(regex_t));
	       /* turn string into a compiled pattern buffer
		* uses POSIX Basic Regular Expressions syntax
		* ignores case
		*/
#ifdef EXT_REGEX
		  if (regcomp(item->preg, tmp, REG_EXTENDED & REG_ICASE & REG_NOSUB) != 0)
#else
		  if (regcomp(item->preg, tmp, REG_ICASE & REG_NOSUB) != 0)
#endif		     
		  {
		     
		  fprintf(stderr, "Failed to make compiled reg exp pattern, ignoring row\n");
		  free(item->preg);
		  free(item);
	       }
	       else {
		  /* set color */
		  item->ansi_color_code = (char *) malloc(sizeof(char) * (strlen(color) + 1));
		  strcpy(item->ansi_color_code, color);
		  /* set callback fkn */
		  item->pf = fkn;
		  /* set param to callback fkn */
		  item->param_to_callback_fkn = fkn_param;

		  /* add to global items array */
		  items[nr++] = item;
	       }
	    }
	 }
      }
   }

   return nr;
}


/* callback function for beep action */
void beep(char *param) 
{
   printf("\a");
}








