/*  colortail -- output last part of file(s) in color
    Copyright (C) 1999  Joakim Andersson <pt98jan@student.hk-r.se>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "main.h"
#include "cfg.h"
#include "read.h"

#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <signal.h>
#include <pwd.h>

int main(int argc, char **argv) 
{
	int follow = 0, color = 1, rows = 10, verbose = 1, cfg_file = 0, i, c;
	char *str;
	struct passwd *pwd = NULL;
	
	init_globals();

	signal(SIGINT, clean_up);
	signal(SIGHUP, clean_up);
	signal(SIGKILL, clean_up);

	/* malloc memory for files to tail, can be max
	   argc - 2 files */
	files = (FILE **) malloc(sizeof(FILE*) * (argc - 2));

	/* malloc memory for the file names, can be max argc - 2 files */
	file_names = (char **) malloc(sizeof(char*) * (argc - 2));
	
	if (argc < 2) {
		print_usage();
		clean_up();
	}

	while(1) {
		c = getopt(argc, argv, "qvn:fk:l");
		if (c == -1)
			break;
		switch (c) {
		case 'v':
			print_info();
			clean_up();
			break;
		case 'f':
			follow = 1;
			break;
		case 'k':
			cfg_file = 1;
			if ((cfgfile = fopen(optarg, "r")) == NULL) {
				fprintf(stderr,
					"Can't open config file: %s\n",
					optarg);
				clean_up();
			}
			break;
		case 'l':
			color = 0;
			cfg_file = 0;
			break;
		case 'n':
			if (check_digits(optarg) == 0) {
				rows = atoi(optarg);
			} else {
				printf("Enter a number as number of rows\n");
				clean_up();
			}
			break;
		case 'q':
			verbose = 0;
			break;
		default:
			printf("Unrecognized option %c\n", c);
			print_usage();
			clean_up();
		}
	}
	
	while (optind < argc) {
		if ((files[nfiles] = fopen(argv[optind], "r")) == NULL) {
			fprintf(stderr, "Can't open tail file: %s\n",
				argv[optind]);
			clean_up();
				
		}
		file_names[nfiles] = (char*)
			malloc(sizeof(char) * (strlen(argv[optind]) + 1));
		strcpy(file_names[nfiles], argv[optind]);
       
		optind++;
		nfiles++;
	}

	if (nfiles == 0) {
		print_usage();
		clean_up();
	}

	if (cfg_file == 0 && color == 1) {
		pwd = getpwuid(getuid());
		if (pwd) {
			int len = strlen(pwd->pw_dir) + strlen(".colortail") + 1 + 1;
			char *tmp = malloc(sizeof(char) * len);
			snprintf(tmp, len,
				 "%s/.colortail", pwd->pw_dir);

			/* try to open cfgfile in home directory */
			if ((cfgfile = fopen(tmp, "r")) == NULL) {
				/* try to open global cfgfile */
				cfgfile = fopen("/etc/colortail", "r");
				
/*				fprintf(stderr,
				"Can't open config file: %s\n", tmp); */
				color = 0;
				/*clean_up();*/
			}
			free(tmp);
		} else {
			color = 0;
		}
	}
	

	if (color)
		nr_items = parse_cfgfile(cfgfile);
	
	if (nfiles == 1) {
		/* only one file */
		if (find_nth_return_from_end(files[0], rows) != 0) {
			printf("File error (find_nth_return)\n");
			clean_up();
		}
		read_and_colorize(files[0], nr_items);
	} else {
		/* more than one file */
		for (i = 0 ; i < nfiles ; i++) {
			if (verbose) {
				if (rows != 0) {
					printf("--> %s <--\n", file_names[i]);
				}
			}
			if (find_nth_return_from_end(files[i], rows) != 0) {
				printf("File error\n");
				clean_up();
			}
			read_and_colorize(files[i], nr_items);
		}
	}
	
	
	if (follow == 1) {
		tail_forever(files, nfiles, verbose);
	}
	
	/* clean up */
	clean_up();
	
	return 0;
}

void free_items(int nr) {
   /* deletes the items, frees the memory */
   int i;

   for (i = 0 ; i < nr ; i++) {
      free(items[i]->preg);
      free(items[i]->ansi_color_code);
      if (items[i]->param_to_callback_fkn)
	      free(items[i]->param_to_callback_fkn);
      free(items[i]);
   }
}

void clean_up()
{ /* general cleanup function */
	int i;
	
	if (items) {
		if (nr_items)
			free_items(nr_items);
	}
	
	if (cfgfile)
		fclose(cfgfile);

	if (files) {
		for (i = 0 ; i < nfiles ; i++) {
			fclose(files[i]);
		}
		free(files);
	}

	if (file_names) {
		for (i = 0 ; i < nfiles ; i++) {
			free(file_names[i]);
		}
		free(file_names);
	}

	if (fds)
		free(fds);

	if (file_sizes)
		free(file_sizes);
	
	exit(0);
}

/* for nfiles > 1 number of files */
void tail_forever(FILE **files, int nfiles, int verbose)
{
	int i, changed, last_changed;
	struct stat stats;

	fds = malloc(sizeof(int) * nfiles);
	file_sizes = malloc(sizeof(off_t) * nfiles);

	/* init file size array with current file sizes */
	for (i = 0 ; i < nfiles ; i++) {
		fds[i] = fileno(files[i]);
		if (fstat(fds[i], &stats) < 0) {
			printf("Major error in tail_forever\n");
			clean_up();
		}
		file_sizes[i] = stats.st_size;
	}

	/* loop forever */
	last_changed = -1;
	while (1) {
		changed = 0;

		for (i = 0 ; i < nfiles ; i++) {
			if (fstat(fds[i], &stats) < 0) {
				printf("Major error in tail_forever\n");
				clean_up();
			}
			/* larger */
			if (file_sizes[i] < stats.st_size) {
				changed = 1;
				if (i != last_changed) {
					if (verbose != 0) {
						printf("--> %s <--\n", file_names[i]);
					}
					last_changed = i;
				}
				file_sizes[i] +=
					read_and_colorize(files[i], nr_items);
			}
			/* truncated */
			else if (file_sizes[i] > stats.st_size) {
				changed = 1;
				/* seek end of file */
				fseek(files[i], stats.st_size, SEEK_SET);
				file_sizes[i] = stats.st_size;
			}
		}

		/* if none of the files changed size, sleep */
		if (!changed)
			sleep(1);
	}
}

void init_globals() 
{
	nr_items = 0;

	cfgfile = NULL;
	files = NULL;
	file_names = NULL;
	nfiles = 0;

	fds = NULL;
	file_sizes = NULL;
}

int check_digits(char *str) 
{
	int i = 0, len = strlen(str);

	for (i = 0 ; i < len ; i++) {
		if (str[i] < '0' && str[i] > '9')
			return 1;
	}
	return 0;
}

   


