/*  colortail -- output last part of file(s) in color
    Copyright (C) 1999  Joakim Andersson <pt98jan@student.hk-r.se>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "main.h"
#include "read.h"
#include "compare.h"

#include <string.h>
#include <regex.h>

/* start from current file position and read och colorize to the
   end of the file.
   Returns number of bytes read.
*/
long read_and_colorize(FILE *file, int nitems)
{
	char tmp[MAX_READ_LINE];
	int i, found, eof = 0;
	long tot = 0;

	/* loop forever */
	while (1) {
		/* read to a '\n' or EOF but max MAX_READ_LINE chars */
		for (i = 0 ; i < MAX_READ_LINE - 2 ; i++) {
			/* read next char */
			tmp[i] = fgetc(file);
			/* are we at the end of the file */
			if (tmp[i] == EOF) {
				/* was the first char read the EOF */
				if (i == 0)
					return tot;
				eof = 1;
				break;
			}
			/* increase total bytes counter */
			tot++;
			/* was it a '\n' char */
			if (tmp[i] == '\n') {
				break;
			}
		}
		/* null terminate the array of chars */
		tmp[i] = '\0';
	
		found = 0;
		/* go through all the items */
		for (i = 0 ; i < nitems ; i++) {
			/* search for search_string in tmp */
/*			if (compare(tmp, items[i]->search_string) == 0) { */
			if (regexec(items[i]->preg, tmp, (size_t) 0, NULL, 0) == 0) {
				/* found it, print the chars in the
				   color from the item */
				printf("%s%s%s\n",
				       items[i]->ansi_color_code, tmp,
				       ANSI_RESET_STR);
				found = 1;
				/* check if there is a callback function to be called */
				if (items[i]->pf) 
				{
				   items[i]->pf(items[i]->param_to_callback_fkn);
				}
				break;
			}
		}

		/* did we find tmp among the items */
		if (found == 0) {
			/* no, then print without color */
			printf("%s\n", tmp);
		}

		/* flush the stdout stream, to make sure the text is printed */
		fflush(stdout);

		/* did we read the EOF */
		if (eof) {
			/* return total bytes read */
			return tot;
		}
	}

	/* will never get here */
	return tot;
}

/* find the nth \n from the end of the file. Returns:
   0 - sucess
   1 - no file (file == NULL)
   2 - fseek error
*/
int find_nth_return_from_end(FILE *file, int n) 
{
	char tmp[BUFSIZE];
	int i, j = 0, size_to_read, loop;
	long curpos;

	if (file == NULL) {
		/* no file, return 1 */
		return 1;
	}
	
	/* find end of file */
	if (fseek(file, 0, SEEK_END) != 0)
		return 2;

	/* if number of returns to find is zero */
	if (n == 0) {
		return 0;
	}
	
	/* set curpos to end of file */
	curpos = ftell(file);

	loop = 1;
	while (loop) {
		/* seek back BUFSIZE from current postition */
		curpos -= BUFSIZE;
		if (curpos < 0) {
			/* negativ means to read before start of file
			   that won't work */
			/* curpos is < 0, so it really
			   is BUFSIZE - abs(curpos) */
			size_to_read = BUFSIZE + curpos;
			/* set curpos to 0, can't seek to before
			   start of file */
			curpos = 0;
			/* end loop, because we are done */
			loop = 0;
		} else {
			/* curpos isn't negativ */
			size_to_read = BUFSIZE;
		}
		
		if (fseek(file, curpos, SEEK_SET) != 0) {
			/* error seeking */
			return 2;
		}
		
		/* read max size_to_read chars from file */
		i = readchars(file, tmp, size_to_read);

		/* compensate for the 0..X thing */
		i--;
		/* go through all chars and look for \n */
		while (i > 0) {
			if (tmp[i] == '\n') {
				/* Found return */
				/* if enough returns */
				if (j == (n-1)) {
					/* seek to char after the return */
  					if (fseek(file,
					curpos + i+1, SEEK_SET) != 0)
						return 2;
					else
						return 0;
				}
				/* increase return counter */
				j++;
				
			}
			/* decrease char counter */
			i--;
		}
	}
	/* at top of file, rewind it, to show all of the file */
	rewind(file);
	return 0;
}

/* Reads max max chars or until EOF from file to str.
   Returns number of chars read.
 */
int readchars(FILE *file, char *str, int max) 
{
	int i = 0;

	for (i = 0 ; i < (max-1) ; i++) {
		str[i] = fgetc(file);
		if (str[i] == EOF) {
			str[i] = '\0';
			return i;
		}
	}
	str[i] = '\0';
	return i;
}

	










